@@ -16,21 +16,20 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from apt.debfile import DebPackage
-import apt_pkg
 import locale
+import logging
 import os
 import re
-import string
 
-from apt import Cache
-from apt import debfile
 from gettext import gettext as _
-from mimetypes import guess_type
+from softwarecenter.backend.channel import is_channel_available
 from softwarecenter.distro import get_distro
-from softwarecenter.enums import *
-from softwarecenter.paths import *
-from softwarecenter.utils import *
+from softwarecenter.enums import PkgStates, XapianValues, Icons
+
+from softwarecenter.paths import (APP_INSTALL_CHANNELS_PATH,
+                                  SOFTWARE_CENTER_ICON_CACHE_DIR,
+                                  )
+from softwarecenter.utils import utf8
 
 LOG = logging.getLogger(__name__)
 
@@ -49,7 +48,7 @@ class Application(object):
         # defaults
         self.pkgname = pkgname.replace("$kernel", os.uname()[2])
         if appname:
-            self.appname = unicode(appname)
+            self.appname = utf8(appname)
         else:
             self.appname = ''
         # the request can take additional "request" data like apturl
@@ -66,7 +65,7 @@ class Application(object):
         """Show user visible name"""
         if self.appname:
             return self.appname
-        return self.pkgname.capitalize()
+        return self.pkgname
     @property
     def popcon(self):
         return self._popcon
@@ -83,7 +82,7 @@ class Application(object):
             doc = db.get_xapian_document(self.appname, self.pkgname)
         except IndexError:
             return self
-        untranslated_application = doc.get_value(XAPIAN_VALUE_APPNAME_UNTRANSLATED)
+        untranslated_application = doc.get_value(XapianValues.APPNAME_UNTRANSLATED)
         uapp = Application(untranslated_application, self.pkgname)
         return uapp
 
@@ -117,7 +116,7 @@ class Application(object):
     def __cmp__(self, other):
         return self.apps_cmp(self, other)
     def __str__(self):
-        return "%s,%s" % (self.appname, self.pkgname)
+        return unicode("%s,%s", 'utf8').encode('utf8') % (self.appname, self.pkgname)
     def __repr__(self):
         return "[Application: appname=%s pkgname=%s]" % (self.appname, self.pkgname)
     @staticmethod
@@ -134,20 +133,6 @@ class Application(object):
         else:
             return cmp(x.pkgname, y.pkgname)
 
-
-class DebFileApplication(Application):
-    def __init__(self, debfile):
-        # deb overrides this
-#        if not debfile.endswith(".deb") and not debfile.count('/') >= 2:
- #           raise ValueError("Need a deb file, got '%s'" % debfile)
-        debname = os.path.splitext(os.path.basename(debfile))[0]
-        self.appname = ""
-        self.pkgname = debname.split('_')[0].lower()
-        self.request = debfile
-    def get_details(self, db):
-        return AppDetailsDebFile(db, application=self)
-
-
 # the details
 class AppDetails(object):
     """ The details for a Application. This contains all the information
@@ -159,7 +144,7 @@ class AppDetails(object):
             a xapian.Document or from a db.application.Application object
         """
         if not doc and not application:
-            raise ValueError, "Need either document or application"
+            raise ValueError("Need either document or application")
         self._db = db
         self._db.connect("reopen", self._on_db_reopen)
         self._cache = self._db._aptcache
@@ -184,7 +169,7 @@ class AppDetails(object):
         # sustitute for apturl
         if self._app.request:
             self._app.request = self._app.request.replace(
-                "$distro", self._distro.get_distro_codename())
+                "$distro", self._distro.get_codename())
 
         # load pkg cache
         self._pkg = None
@@ -210,8 +195,8 @@ class AppDetails(object):
                     not debfile_matches and 
                     not channel_matches and 
                     not section_matches):
-                    self._error = _("Not Found")
-                    self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
+                    self._error = _("Not found")
+                    self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
 
     def same_app(self, other):
         return self.pkgname == other.pkgname
@@ -229,7 +214,7 @@ class AppDetails(object):
     @property
     def channelname(self):
         if self._doc:
-            channel = self._doc.get_value(XAPIAN_VALUE_ARCHIVE_CHANNEL)
+            channel = self._doc.get_value(XapianValues.ARCHIVE_CHANNEL)
             path = APP_INSTALL_CHANNELS_PATH + channel + ".list"
             if os.path.exists(path):
                 return channel
@@ -271,7 +256,7 @@ class AppDetails(object):
                     return origin.component
         # then xapian
         elif self._doc:
-            comp = self._doc.get_value(XAPIAN_VALUE_ARCHIVE_SECTION)
+            comp = self._doc.get_value(XapianValues.ARCHIVE_SECTION)
             return comp
         # then apturl requests
         else:
@@ -287,18 +272,18 @@ class AppDetails(object):
     @property
     def desktop_file(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_DESKTOP_FILE)
+            return self._doc.get_value(XapianValues.DESKTOP_FILE)
 
     @property
     def description(self):
         if self._pkg:
             return self._pkg.candidate.description
         elif self._doc:
-            if self._doc.get_value(XAPIAN_VALUE_SC_DESCRIPTION):
-                return self._doc.get_value(XAPIAN_VALUE_SC_DESCRIPTION)
+            if self._doc.get_value(XapianValues.SC_DESCRIPTION):
+                return self._doc.get_value(XapianValues.SC_DESCRIPTION)
         # if its in need-source state and we have a eula, display it
         # as the description
-        if self.pkg_state == PKG_STATE_NEEDS_SOURCE and self.eulafile:
+        if self.pkg_state == PkgStates.NEEDS_SOURCE and self.eulafile:
             return open(self.eulafile).read()
         return ""
 
@@ -309,33 +294,29 @@ class AppDetails(object):
         elif self._error:
             return self._error
         # this may have changed since we inited the appdetails
-        elif self.pkg_state == PKG_STATE_NOT_FOUND:
-            self._error =  _("Not Found")
-            self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
+        elif self.pkg_state == PkgStates.NOT_FOUND:
+            self._error =  _("Not found")
+            self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
             return self._error_not_found
 
     @property
     def icon(self):
-        if self.pkg_state == PKG_STATE_NOT_FOUND:
-            return MISSING_PKG_ICON
+        if self.pkg_state == PkgStates.NOT_FOUND:
+            return Icons.MISSING_PKG
         if self._doc:
             return os.path.splitext(self._db.get_iconname(self._doc))[0]
         if not self.summary:
-            return MISSING_PKG_ICON
+            return Icons.MISSING_PKG
             
     @property
     def icon_file_name(self):
         if self._doc:
             return self._db.get_iconname(self._doc)
     
-    @property
-    def icon_needs_download(self):
-        if self._doc:
-            return self._db.get_icon_needs_download(self._doc)
-            
     @property
     def icon_url(self):
-        return self._distro.get_downloadable_icon_url(self._cache, self.pkgname, self.icon_file_name)
+        if self._doc:
+            return self._db.get_icon_download_url(self._doc)
 
     @property
     def cached_icon_file_path(self):
@@ -344,18 +325,27 @@ class AppDetails(object):
 
     @property
     def installation_date(self):
-        from softwarecenter.apt.apthistory import get_apt_history
-        self._history = get_apt_history()
+        from softwarecenter.db.history import get_pkg_history
+        self._history = get_pkg_history()
         return self._history.get_installed_date(self.pkgname)
         
     @property
     def purchase_date(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_PURCHASED_DATE)
+            return self._doc.get_value(XapianValues.PURCHASED_DATE)
 
     @property
     def license(self):
-        return self._distro.get_license_text(self.component)
+        xapian_license = None
+        if self._doc:
+            xapian_license = self._doc.get_value(XapianValues.LICENSE)
+        if xapian_license:
+            # try to i18n this, the server side does not yet support
+            # translations, but fortunately for the most common ones
+            # like "Properitary" we have translations in s-c
+            return _(xapian_license)
+        else:
+            return self._distro.get_license_text(self.component)
 
     @property
     def maintenance_status(self):
@@ -406,7 +396,7 @@ class AppDetails(object):
     def pkg_state(self):
         # puchase state
         if self.pkgname in self._backend.pending_purchases:
-            return PKG_STATE_INSTALLING_PURCHASED
+            return PkgStates.INSTALLING_PURCHASED
 
         # via the pending transactions dict
         if self.pkgname in self._backend.pending_transactions:
@@ -414,9 +404,9 @@ class AppDetails(object):
             # if there is no self._pkg yet, that means this is a INSTALL
             # from a previously not-enabled source (like a purchase)
             if self._pkg and self._pkg.installed:
-                return PKG_STATE_REMOVING
+                return PkgStates.REMOVING
             else:
-                return PKG_STATE_INSTALLING
+                return PkgStates.INSTALLING
 
         # if we have _pkg that means its either:
         # - available for download (via sources.list)
@@ -425,11 +415,11 @@ class AppDetails(object):
         if self._pkg:
             # Don't handle upgrades yet
             #if self._pkg.installed and self._pkg._isUpgradable:
-            #    return PKG_STATE_UPGRADABLE
-            if self._pkg.installed:
-                return PKG_STATE_INSTALLED
+            #    return PkgStates.UPGRADABLE
+            if self._pkg.is_installed:
+                return PkgStates.INSTALLED
             else:
-                return PKG_STATE_UNINSTALLED
+                return PkgStates.UNINSTALLED
         # if we don't have a _pkg, then its either:
         #  - its in a unavailable repo
         #  - the repository information is outdated
@@ -439,53 +429,53 @@ class AppDetails(object):
         if not self._pkg:
             if self.channelname:
                 if self._unavailable_channel():
-                    return PKG_STATE_NEEDS_SOURCE
+                    return PkgStates.NEEDS_SOURCE
                 else:
-                    self._error =  _("Not Found")
-                    self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
-                    return PKG_STATE_NOT_FOUND
+                    self._error =  _("Not found")
+                    self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+                    return PkgStates.NOT_FOUND
             else:
                 if self.price:
-                    return PKG_STATE_NEEDS_PURCHASE
+                    return PkgStates.NEEDS_PURCHASE
                 if (self.purchase_date and
-                    self._doc.get_value(XAPIAN_VALUE_ARCHIVE_DEB_LINE)):
-                    return PKG_STATE_PURCHASED_BUT_REPO_MUST_BE_ENABLED
+                    self._doc.get_value(XapianValues.ARCHIVE_DEB_LINE)):
+                    return PkgStates.PURCHASED_BUT_REPO_MUST_BE_ENABLED
                 if self.component:
                     components = self.component.split('&')
                     for component in components:
                         if component and self._unavailable_component(component_to_check=component):
-                            return PKG_STATE_NEEDS_SOURCE
-                self._error =  _("Not Found")
-                self._error_not_found = _("There isn't a software package called \"%s\" in your current software sources.") % self.pkgname.capitalize()
-                return PKG_STATE_NOT_FOUND
-        return PKG_STATE_UNKNOWN
+                            return PkgStates.NEEDS_SOURCE
+                self._error =  _("Not found")
+                self._error_not_found = utf8(_(u"There isn\u2019t a software package called \u201c%s\u201D in your current software sources.")) % utf8(self.pkgname)
+                return PkgStates.NOT_FOUND
+        return PkgStates.UNKNOWN
 
     @property
     def price(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_PRICE)
+            return self._doc.get_value(XapianValues.PRICE)
 
     @property
     def ppaname(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_ARCHIVE_PPA)
+            return self._doc.get_value(XapianValues.ARCHIVE_PPA)
 
     @property
     def deb_line(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_ARCHIVE_DEB_LINE)
+            return self._doc.get_value(XapianValues.ARCHIVE_DEB_LINE)
 
     @property
     def signing_key_id(self):
         if self._doc:
-            return self._doc.get_value(XAPIAN_VALUE_ARCHIVE_SIGNING_KEY_ID)
+            return self._doc.get_value(XapianValues.ARCHIVE_SIGNING_KEY_ID)
 
     @property
     def screenshot(self):
         # if there is a custom screenshot url provided, use that
         if self._doc:
-            if self._doc.get_value(XAPIAN_VALUE_SCREENSHOT_URL):
-                return self._doc.get_value(XAPIAN_VALUE_SCREENSHOT_URL)
+            if self._doc.get_value(XapianValues.SCREENSHOT_URL):
+                return self._doc.get_value(XapianValues.SCREENSHOT_URL)
         # else use the default
         return self._distro.SCREENSHOT_LARGE_URL % { 'pkgname' : self.pkgname, 
                                                      'version' : self.version or 0 }
@@ -501,8 +491,8 @@ class AppDetails(object):
     def thumbnail(self):
         # if there is a custom thumbnail url provided, use that
         if self._doc:
-            if self._doc.get_value(XAPIAN_VALUE_THUMBNAIL_URL):
-                return self._doc.get_value(XAPIAN_VALUE_THUMBNAIL_URL)
+            if self._doc.get_value(XapianValues.THUMBNAIL_URL):
+                return self._doc.get_value(XapianValues.THUMBNAIL_URL)
         # else use the default
         return self._distro.SCREENSHOT_THUMB_URL % { 'pkgname' : self.pkgname, 
                                                      'version' : self.version or 0}
@@ -518,12 +508,13 @@ class AppDetails(object):
     @property
     def warning(self):
         # apturl minver matches
-        if not self.pkg_state == PKG_STATE_INSTALLED:
+        if not self.pkg_state == PkgStates.INSTALLED:
             if self._app.request:
                 minver_matches = re.findall(r'minver=[a-z,0-9,-,+,.,~]*', self._app.request)
                 if minver_matches and self.version:
                     minver = minver_matches[0][7:]
-                    if apt_pkg.version_compare(minver, self.version) > 0:
+                    from softwarecenter.utils import version_compare
+                    if version_compare(minver, self.version) > 0:
                         return _("Version %s or later not available.") % minver
         # can we enable a source
         if not self._pkg:
@@ -552,13 +543,21 @@ class AppDetails(object):
     @property
     def website(self):
         if self._pkg:
-            return self._pkg.candidate.homepage
+            return self._pkg.website
+    
+    @property
+    def license_key(self):
+        if self._doc:
+            return self._doc.get_value(XapianValues.LICENSE_KEY)
+
+    @property
+    def license_key_path(self):
+        if self._doc:
+            return self._doc.get_value(XapianValues.LICENSE_KEY_PATH)
 
     def _unavailable_channel(self):
         """ Check if the given doc refers to a channel that is currently not enabled """
-        p = os.path.join(apt_pkg.config.find_dir("Dir::Etc::sourceparts"),
-                         "%s.list" % self.channelname)
-        return not os.path.exists(p)
+        return not is_channel_available(self.channelname)
 
     def _unavailable_component(self, component_to_check=None):
         """ Check if the given doc refers to a component that is currently not enabled """
@@ -567,7 +566,7 @@ class AppDetails(object):
         elif self.component:
             component = self.component
         else:
-            component =  self._doc.get_value(XAPIAN_VALUE_ARCHIVE_SECTION)
+            component =  self._doc.get_value(XapianValues.ARCHIVE_SECTION)
         if not component:
             return False
         distro_codename = self._distro.get_codename()
@@ -590,7 +589,6 @@ class AppDetails(object):
         details.append("               error: %s" % self.error)
         details.append("                icon: %s" % self.icon)
         details.append("      icon_file_name: %s" % self.icon_file_name)
-        details.append(" icon_needs_download: %s" % self.icon_needs_download)
         details.append("            icon_url: %s" % self.icon_url)
         details.append("   installation_date: %s" % self.installation_date)
         details.append("       purchase_date: %s" % self.purchase_date)
@@ -605,142 +603,3 @@ class AppDetails(object):
         details.append("             version: %s" % self.version)
         details.append("             website: %s" % self.website)
         return '\n'.join(details)
-
-class AppDetailsDebFile(AppDetails):
-    
-    def __init__(self, db, doc=None, application=None):
-        super(AppDetailsDebFile, self).__init__(db, doc, application)
-        if doc:
-            raise ValueError("doc must be None for deb files")
-
-        try:
-            # for some reason Cache() is much faster than "self._cache._cache"
-            # on startup
-            self._deb = DebPackage(self._app.request, Cache())
-        except:
-            self._deb = None
-            self._pkg = None
-            if not os.path.exists(self._app.request):
-                self._error = _("Not Found")
-                self._error_not_found = _("The file \"%s\" does not exist.") % self._app.request
-            else:
-                mimetype = guess_type(self._app.request)
-                if mimetype[0] != "application/x-debian-package":
-                    self._error =  _("Not Found")
-                    self._error_not_found = _("The file \"%s\" is not a software package.") % self._app.request
-                else:
-                    # hm, deb files from launchpad get this error..
-                    self._error =  _("Internal Error")
-                    self._error_not_found = _("The file \"%s\" could not be opened.") % self._app.request
-            return
-
-        if self.pkgname and self.pkgname != self._app.pkgname:
-            # this happens when the deb file has a quirky file name
-            self._app.pkgname = self.pkgname
-
-            # load pkg cache
-            self._pkg = None
-            if (self._app.pkgname in self._cache and 
-                self._cache[self._app.pkgname].candidate):
-                self._pkg = self._cache[self._app.pkgname]
-            # load xapian document
-            self._doc = None
-            try:
-                self._doc = self._db.get_xapian_document(
-                    self._app.appname, self._app.pkgname)
-            except:
-                pass
-
-        # check deb and set failure state on error
-        if not self._deb.check():
-            self._error = self._deb._failure_string
-
-    @property
-    def description(self):
-        if self._deb:
-            description = self._deb._sections["Description"]
-            return ('\n').join(description.split('\n')[1:]).replace(" .\n", "")
-        return ""
-
-    @property
-    def maintenance_status(self):
-        return None
-
-    @property
-    def pkgname(self):
-        if self._deb:
-            return self._deb._sections["Package"]
-
-    @property
-    def pkg_state(self):
-        if self._error:
-            if self._error_not_found:
-                return PKG_STATE_NOT_FOUND
-            else:
-                return PKG_STATE_ERROR
-        if self._deb:
-            deb_state = self._deb.compare_to_version_in_cache()
-            if deb_state == DebPackage.VERSION_NONE:
-                return PKG_STATE_UNINSTALLED
-            elif deb_state == DebPackage.VERSION_OUTDATED:
-                if self._cache[self.pkgname].installed:
-                    return PKG_STATE_INSTALLED
-                else:
-                    return PKG_STATE_UNINSTALLED
-            elif deb_state == DebPackage.VERSION_SAME:
-                return PKG_STATE_REINSTALLABLE
-            elif deb_state == DebPackage.VERSION_NEWER:
-                if self._cache[self.pkgname].installed:
-                    return PKG_STATE_UPGRADABLE
-                else:
-                    return PKG_STATE_UNINSTALLED
-    
-    @property
-    def summary(self):
-        if self._deb:
-            description = self._deb._sections["Description"]
-            return description.split('\n')[0]
-
-    @property
-    def display_summary(self):
-        if self._doc:
-            name = self._db.get_appname(self._doc)
-            if name:
-                return self.summary
-            else:
-                # by spec..
-                return self._db.get_pkgname(self._doc)
-        return self.summary
-
-    @property
-    def version(self):
-        if self._deb:
-            return self._deb._sections["Version"]
-
-    @property
-    def warning(self):
-        # FIXME: use more concise warnings
-        if self._deb:
-            deb_state = self._deb.compare_to_version_in_cache(use_installed=False)
-            if deb_state == DebPackage.VERSION_NONE:
-                return _("Only install this file if you trust the origin.")
-            elif (not self._cache[self.pkgname].installed and
-                  self._cache[self.pkgname].candidate and
-                  self._cache[self.pkgname].candidate.downloadable): 
-                if deb_state == DebPackage.VERSION_OUTDATED:
-                    return _("Please install \"%s\" via your normal software channels. Only install this file if you trust the origin.") % self.name
-                elif deb_state == DebPackage.VERSION_SAME:
-                    return _("Please install \"%s\" via your normal software channels. Only install this file if you trust the origin.") % self.name
-                elif deb_state == DebPackage.VERSION_NEWER:
-                    return _("An older version of \"%s\" is available in your normal software channels. Only install this file if you trust the origin.") % self.name
-
-    @property
-    def website(self):
-        if self._deb:
-            website = None
-            try:
-                website = self._deb._sections["Homepage"]
-            except:
-                pass
-            if website:
-                return website
