@@ -17,32 +17,53 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import apt
-import apt_pkg
 import base64
-import glib
 import logging
 import os
-import simplejson
+import json
 import string
 import shutil
-import sys
 import time
-import urllib
 import xapian
 
-from ConfigParser import RawConfigParser, NoOptionError
+from gi.repository import GObject
+
+from softwarecenter.utils import utf8
+
+# py3 compat
+try:
+    from configparser import RawConfigParser, NoOptionError
+    RawConfigParser # pyflakes
+    NoOptionError # pyflakes
+except ImportError:
+    from ConfigParser import RawConfigParser, NoOptionError
+
+# py3 compat
+try:
+    import cPickle as pickle
+    pickle # pyflakes
+except ImportError:
+    import pickle
+
+
 from gettext import gettext as _
 from glob import glob
 
-from softwarecenter.paths import *
-from softwarecenter.enums import *
-from softwarecenter.paths import SOFTWARE_CENTER_ICON_CACHE_DIR
+import softwarecenter.paths
+
+from softwarecenter.enums import (XapianValues,
+                                  DB_SCHEMA_VERSION,
+                                  AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME,
+                                  PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME,
+                                  )
 from softwarecenter.db.database import parse_axi_values_file
 
 from locale import getdefaultlocale
 import gettext
-import cPickle
+
+
+from softwarecenter.db.pkginfo import get_pkg_info
+from softwarecenter.distro import get_current_arch, get_foreign_architectures
 
 # weights for the different fields
 WEIGHT_DESKTOP_NAME = 10
@@ -67,11 +88,14 @@ cataloged_times = {}
 CF = "/var/lib/apt-xapian-index/cataloged_times.p"
 if os.path.exists(CF):
     try:
-        cataloged_times = cPickle.load(open(CF))
+        cataloged_times = pickle.load(open(CF))
     except EOFError as e:
-        logging.warn("failed to read %s (%s" % (CF, e))
+        LOG.warn("failed to read %s (%s" % (CF, e))
 del CF
 
+# Enable Xapian's CJK tokenizer (see LP: #745243)
+os.environ['XAPIAN_CJK_NGRAM'] = '1'
+
 class AppInfoParserBase(object):
     """ base class for reading AppInfo meta-data """
 
@@ -119,11 +143,15 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
                 'Channel'    : 'channel',
                 'Deb-Line'   : 'deb_line',
                 'Signing-Key-Id' : 'signing_key_id',
+                'License'    : 'license',
                 'Purchased-Date' : 'purchase_date',
+                'License-Key' : 'license_key',
+                'License-Key-Path' : 'license_key_path',
                 'PPA'        : 'archive_id',
                 'Icon'       : 'icon',
                 'Screenshot-Url' : 'screenshot_url',
                 'Thumbnail-Url' : 'thumbnail_url',
+                'Icon-Url'   : 'icon_url',
               }
 
     # map from requested key to a static data element
@@ -135,11 +163,12 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
         self.origin = "software-center-agent"
         self._apply_exceptions()
     def _apply_exceptions(self):
-        # map screenshot to thumbnail
+        # for items from the agent, we use the full-size screenshot for
+        # the thumbnail and scale it for display, this is done because
+        # we no longer keep thumbnail versions of screenshots on the server
         if (hasattr(self.sca_entry, "screenshot_url") and 
             not hasattr(self.sca_entry, "thumbnail_url")):
-            url = self.sca_entry.screenshot_url.replace(".png", ".thumb.png")
-            self.sca_entry.thumbnail_url = url
+            self.sca_entry.thumbnail_url = self.sca_entry.screenshot_url
         if hasattr(self.sca_entry, "description"):
             self.sca_entry.Comment = self.sca_entry.description.split("\n")[0]
             self.sca_entry.Description = "\n".join(self.sca_entry.description.split("\n")[1:])
@@ -147,6 +176,11 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
         if key in self.STATIC_DATA:
             return self.STATIC_DATA[key]
         return getattr(self.sca_entry, self._apply_mapping(key))
+    def get_desktop_categories(self):
+        try:
+            return ['DEPARTMENT:' + self.sca_entry.department[-1]] + self._get_desktop_list("Categories")
+        except:
+            return self._get_desktop_list("Categories")
     def has_option_desktop(self, key):
         return (key in self.STATIC_DATA or
                 hasattr(self.sca_entry, self._apply_mapping(key)))
@@ -190,17 +224,28 @@ class AppStreamXMLParser(AppInfoParserBase):
               "keywords"  : "keyword",
               "mimetypes" : "mimetype",
             }
-    
+
+    # map from requested key to a static data element
+    STATIC_DATA = { 'Type' : 'Application',
+                  }
+
     def __init__(self, appinfo_xml, xmlfile):
         self.appinfo_xml = appinfo_xml
         self.xmlfile = xmlfile
     def get_desktop(self, key, translated=True):
-        from lxml import etree
+        if key in self.STATIC_DATA:
+            return self.STATIC_DATA[key]
         key = self._apply_mapping(key)
         if key in self.LISTS:
             return self._parse_with_lists(key)
         else:
             return self._parse_value(key)
+    def get_desktop_categories(self):
+        return self._get_desktop_list("Categories", split_str=',')
+    def get_desktop_mimetypes(self):
+        if not self.has_option_desktop("MimeType"):
+            return []
+        return self._get_desktop_list("MimeType", split_str=',')
     def _parse_value(self, key):
         for child in self.appinfo_xml.iter(key):
             # FIXME: deal with the i18n
@@ -215,7 +260,8 @@ class AppStreamXMLParser(AppInfoParserBase):
                 l.append(child.text)
         return ",".join(l)
     def has_option_desktop(self, key):
-        from lxml import etree
+        if key in self.STATIC_DATA: 
+            return True
         key = self._apply_mapping(key)
         return not self.appinfo_xml.find(key) is None
     @property
@@ -254,7 +300,7 @@ class DesktopTagSectionParser(AppInfoParserBase):
                     locale_short = locale.split("_")[0]
                     if self.has_option_desktop("%s-%s" % (key, locale_short)):
                         return self.tag_section["%s-%s" % (key, locale_short)]
-        except ValueError,e :
+        except ValueError:
             pass
         # and then the untranslated field
         return self.tag_section[key]
@@ -286,6 +332,12 @@ class DesktopConfigParser(RawConfigParser, AppInfoParserBase):
                 translated_value = gettext.dgettext(domain, value)
                 if value != translated_value:
                     return translated_value
+        # then try app-install-data 
+        value = self.get(self.DE, key)
+        if value:
+            translated_value = gettext.dgettext("app-install-data", value)
+            if value != translated_value:
+                return translated_value
         # then try the i18n version of the key (in [de_DE] or
         # [de]) but ignore errors and return the untranslated one then
         try:
@@ -297,7 +349,7 @@ class DesktopConfigParser(RawConfigParser, AppInfoParserBase):
                     locale_short = locale.split("_")[0]
                     if self.has_option_desktop("%s[%s]" % (key, locale_short)):
                         return self.get(self.DE, "%s[%s]" % (key, locale_short))
-        except ValueError,e :
+        except ValueError:
             pass
         # and then the untranslated field
         return self.get(self.DE, key)
@@ -320,12 +372,14 @@ def ascii_upper(key):
 
 def index_name(doc, name, term_generator):
     """ index the name of the application """
-    doc.add_value(XAPIAN_VALUE_APPNAME, name)
+    doc.add_value(XapianValues.APPNAME, name)
     doc.add_term("AA"+name)
     w = globals()["WEIGHT_DESKTOP_NAME"]
     term_generator.index_text_without_positions(name, w)
 
-def update(db, cache, datadir=APP_INSTALL_PATH):
+def update(db, cache, datadir=None):
+    if not datadir:
+        datadir = softwarecenter.paths.APP_INSTALL_DESKTOP_PATH
     update_from_app_install_data(db, cache, datadir)
     update_from_var_lib_apt_lists(db, cache)
     # add db global meta-data
@@ -335,16 +389,20 @@ def update(db, cache, datadir=APP_INSTALL_PATH):
 def update_from_json_string(db, cache, json_string, origin):
     """ index from a json string, should include origin url (free form string)
     """
-    for sec in simplejson.loads(json_string):
+    for sec in json.loads(json_string):
         parser = JsonTagSectionParser(sec, origin)
         index_app_info_from_parser(parser, db, cache)
     return True
 
 def update_from_var_lib_apt_lists(db, cache, listsdir=None):
     """ index the files in /var/lib/apt/lists/*AppInfo """
+    try:
+        import apt_pkg
+    except ImportError:
+        return False
     if not listsdir:
         listsdir = apt_pkg.Config.find_dir("Dir::State::lists")
-    context = glib.main_context_default()
+    context = GObject.main_context_default()
     for appinfo in glob("%s/*AppInfo" % listsdir):
         LOG.debug("processing %s" % appinfo)
         # process events
@@ -356,9 +414,11 @@ def update_from_var_lib_apt_lists(db, cache, listsdir=None):
             index_app_info_from_parser(parser, db, cache)
     return True
 
-def update_from_appstream_xml(db, cache, xmldir):
+def update_from_appstream_xml(db, cache, xmldir=None):
+    if not xmldir:
+        xmldir = softwarecenter.paths.APPSTREAM_XML_PATH
     from lxml import etree
-    context = glib.main_context_default()
+    context = GObject.main_context_default()
     for appstream_xml in glob(os.path.join(xmldir, "*.xml")):
         LOG.debug("processing %s" % appstream_xml)
         # process events
@@ -367,17 +427,19 @@ def update_from_appstream_xml(db, cache, xmldir):
         tree = etree.parse(open(appstream_xml))
         root = tree.getroot()
         if not root.tag == "applications":
-            logging.error("failed to read '%s' excepected Applications root tag" % appstream_xml)
+            LOG.error("failed to read '%s' expected Applications root tag" % appstream_xml)
             continue
         for appinfo in root.iter("application"):
             parser = AppStreamXMLParser(appinfo, appstream_xml)
             index_app_info_from_parser(parser, db, cache)
     return True
         
-def update_from_app_install_data(db, cache, datadir=APP_INSTALL_PATH):
+def update_from_app_install_data(db, cache, datadir=None):
     """ index the desktop files in $datadir/desktop/*.desktop """
-    context = glib.main_context_default()
-    for desktopf in glob(datadir+"/desktop/*.desktop"):
+    if not datadir:
+        datadir = softwarecenter.paths.APP_INSTALL_DESKTOP_PATH
+    context = GObject.main_context_default()
+    for desktopf in glob(datadir+"/*.desktop"):
         LOG.debug("processing %s" % desktopf)
         # process events
         while context.pending():
@@ -386,7 +448,7 @@ def update_from_app_install_data(db, cache, datadir=APP_INSTALL_PATH):
             parser = DesktopConfigParser()
             parser.read(desktopf)
             index_app_info_from_parser(parser, db, cache)
-        except Exception, e:
+        except Exception as e:
             # Print a warning, no error (Debian Bug #568941)
             LOG.debug("error processing: %s %s" % (desktopf, e))
             warning_text = _(
@@ -431,10 +493,10 @@ def add_from_purchased_but_needs_reinstall_data(purchased_but_may_need_reinstall
             # WARNING: item.name needs to be different than
             #          the item.name in the DB otherwise the DB
             #          gets confused about (appname, pkgname) duplication
-            item.name = _("%s (already purchased)") % item.name
+            item.name = utf8(_("%s (already purchased)")) % utf8(item.name)
             parser = SoftwareCenterAgentParser(item)
             index_app_info_from_parser(parser, db_purchased, cache)
-        except Exception, e:
+        except Exception as e:
             LOG.exception("error processing: %s " % e)
     # add new in memory db to the main db
     db.add_database(db_purchased)
@@ -442,29 +504,38 @@ def add_from_purchased_but_needs_reinstall_data(purchased_but_may_need_reinstall
     query = xapian.Query("AH"+PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME)
     return query
 
-def update_from_software_center_agent(db, cache, ignore_etag=False,
-                                      include_approved_but_unpublished=False):
+def update_from_software_center_agent(db, cache, ignore_cache=False,
+                                      include_sca_qa=False):
     """ update index based on the software-center-agent data """
     def _available_cb(sca, available):
         # print "available: ", available
         LOG.debug("available: '%s'" % available)
         sca.available = available
+        sca.good_data = True
+        loop.quit()
     def _error_cb(sca, error):
         LOG.warn("error: %s" % error)
         sca.available = []
+        sca.good_data = False
+        loop.quit()
     # use the anonymous interface to s-c-agent, scales much better and is
     # much cache friendlier
-    from softwarecenter.backend.restfulclient import SoftwareCenterAgentAnonymous
-    sca = SoftwareCenterAgentAnonymous(ignore_etag)
+    from softwarecenter.backend.scagent import SoftwareCenterAgent
+    # FIXME: honor ignore_etag here somehow with the new piston based API
+    sca = SoftwareCenterAgent(ignore_cache)
     sca.connect("available", _available_cb)
     sca.connect("error", _error_cb)
     sca.available = None
-    sca.query_available(include_approved_but_unpublished)
-    context = glib.main_context_default()
-    while sca.available is None:
-        while context.pending():
-            context.iteration()
-        time.sleep(0.1)
+    if include_sca_qa:
+        sca.query_available_qa()
+    else:
+        sca.query_available()
+    # create event loop and run it until data is available 
+    # (the _available_cb and _error_cb will quit it)
+    context = GObject.main_context_default()
+    loop = GObject.MainLoop(context)
+    loop.run()
+    # process data
     for entry in sca.available:
         # process events
         while context.pending():
@@ -482,18 +553,20 @@ def update_from_software_center_agent(db, cache, ignore_etag=False,
                 icondata = ""
             # write it if we have data
             if icondata:
-		# the iconcache gets mightly confused if there is a "." in the name
+            # the iconcache gets mightly confused if there is a "." in the name
                 iconname = "sc-agent-%s" % entry.package_name.replace(".", "__")
-                open(os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR,
-                                  "%s.png" % iconname),"w").write(icondata)
+                open(os.path.join(
+                        softwarecenter.paths.SOFTWARE_CENTER_ICON_CACHE_DIR,
+                        "%s.png" % iconname),"w").write(icondata)
                 entry.icon = iconname
             # now the normal parser
             parser = SoftwareCenterAgentParser(entry)
             index_app_info_from_parser(parser, db, cache)
-        except Exception, e:
+        except Exception as e:
             LOG.warning("error processing: %s " % e)
-    # return true if we have data entries
-    return len(sca.available) > 0
+    # return true if we have updated entries (this can also be an empty list)
+    # but only if we did not got a error from the agent
+    return sca.good_data
         
 def index_app_info_from_parser(parser, db, cache):
         term_generator = xapian.TermGenerator()
@@ -513,7 +586,10 @@ def index_app_info_from_parser(parser, db, cache):
         doc = xapian.Document()
         term_generator.set_document(doc)
         # app name is the data
-        if parser.has_option_desktop("X-GNOME-FullName"):
+        if parser.has_option_desktop("X-Ubuntu-Software-Center-Name"):
+            name = parser.get_desktop("X-Ubuntu-Software-Center-Name")
+            untranslated_name = parser.get_desktop("X-Ubuntu-Software-Center-Name", translated=False)
+        elif parser.has_option_desktop("X-GNOME-FullName"):
             name = parser.get_desktop("X-GNOME-FullName")
             untranslated_name = parser.get_desktop("X-GNOME-FullName", translated=False)
         else:
@@ -521,10 +597,11 @@ def index_app_info_from_parser(parser, db, cache):
             untranslated_name = parser.get_desktop("Name", translated=False)
         if name in seen:
             LOG.debug("duplicated name '%s' (%s)" % (name, parser.desktopf))
+        LOG.debug("indexing app '%s'" % name)
         seen.add(name)
         doc.set_data(name)
         index_name(doc, name, term_generator)
-        doc.add_value(XAPIAN_VALUE_APPNAME_UNTRANSLATED, untranslated_name)
+        doc.add_value(XapianValues.APPNAME_UNTRANSLATED, untranslated_name)
 
         # check if we should ignore this file
         if parser.has_option_desktop("X-AppInstall-Ignore"):
@@ -532,14 +609,24 @@ def index_app_info_from_parser(parser, db, cache):
             if ignore.strip().lower() == "true":
                 LOG.debug("X-AppInstall-Ignore found for '%s'" % parser.desktopf)
                 return
+        # architecture
+        pkgname_extension = ''
+        if parser.has_option_desktop("X-AppInstall-Architectures"):
+            arches = parser.get_desktop("X-AppInstall-Architectures")
+            doc.add_value(XapianValues.ARCHIVE_ARCH, arches)
+            native_archs = get_current_arch() in arches.split(',')
+            foreign_archs = list(set(arches.split(',')) & set(get_foreign_architectures()))
+            if not (native_archs or foreign_archs): return
+            if not native_archs and foreign_archs:
+                pkgname_extension = ':' + foreign_archs[0]
         # package name
-        pkgname = parser.get_desktop("X-AppInstall-Package")
+        pkgname = parser.get_desktop("X-AppInstall-Package") + pkgname_extension
         doc.add_term("AP"+pkgname)
         if '-' in pkgname:
             # we need this to work around xapian oddness
             doc.add_term(pkgname.replace('-','_'))
-        doc.add_value(XAPIAN_VALUE_PKGNAME, pkgname)
-        doc.add_value(XAPIAN_VALUE_DESKTOP_FILE, parser.desktopf)
+        doc.add_value(XapianValues.PKGNAME, pkgname)
+        doc.add_value(XapianValues.DESKTOP_FILE, parser.desktopf)
         # display name
         if "display_name" in axi_values:
             doc.add_value(axi_values["display_name"], name)
@@ -556,57 +643,77 @@ def index_app_info_from_parser(parser, db, cache):
         if parser.has_option_desktop("X-AppInstall-Section"):
             archive_section = parser.get_desktop("X-AppInstall-Section")
             doc.add_term("AS"+archive_section)
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_SECTION, archive_section)
+            doc.add_value(XapianValues.ARCHIVE_SECTION, archive_section)
         # section (mail, base, ..)
         if pkgname in cache and cache[pkgname].candidate:
-            section = cache[pkgname].candidate.section
+            section = cache[pkgname].section
             doc.add_term("AE"+section)
         # channel (third party stuff)
         if parser.has_option_desktop("X-AppInstall-Channel"):
             archive_channel = parser.get_desktop("X-AppInstall-Channel")
             doc.add_term("AH"+archive_channel)
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_CHANNEL, archive_channel)
-        # singing key (third party)
+            doc.add_value(XapianValues.ARCHIVE_CHANNEL, archive_channel)
+        # signing key (third party)
         if parser.has_option_desktop("X-AppInstall-Signing-Key-Id"):
             keyid = parser.get_desktop("X-AppInstall-Signing-Key-Id")
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_SIGNING_KEY_ID, keyid)
+            doc.add_value(XapianValues.ARCHIVE_SIGNING_KEY_ID, keyid)
+        # license (third party)
+        if parser.has_option_desktop("X-AppInstall-License"):
+            license = parser.get_desktop("X-AppInstall-License")
+            doc.add_value(XapianValues.LICENSE, license)
         # purchased date
         if parser.has_option_desktop("X-AppInstall-Purchased-Date"):
             date = parser.get_desktop("X-AppInstall-Purchased-Date")
             # strip the subseconds from the end of the date string
-            doc.add_value(XAPIAN_VALUE_PURCHASED_DATE, str(date).split(".")[0])
+            doc.add_value(XapianValues.PURCHASED_DATE, str(date).split(".")[0])
         # deb-line (third party)
         if parser.has_option_desktop("X-AppInstall-Deb-Line"):
             debline = parser.get_desktop("X-AppInstall-Deb-Line")
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_DEB_LINE, debline)
+            doc.add_value(XapianValues.ARCHIVE_DEB_LINE, debline)
+        # license key (third party)
+        if parser.has_option_desktop("X-AppInstall-License-Key"):
+            key = parser.get_desktop("X-AppInstall-License-Key")
+            doc.add_value(XapianValues.LICENSE_KEY, key)
+        # license keypath (third party)
+        if parser.has_option_desktop("X-AppInstall-License-Key-Path"):
+            path = parser.get_desktop("X-AppInstall-License-Key-Path")
+            doc.add_value(XapianValues.LICENSE_KEY_PATH, path)
         # PPA (third party stuff)
         if parser.has_option_desktop("X-AppInstall-PPA"):
             archive_ppa = parser.get_desktop("X-AppInstall-PPA")
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_PPA, archive_ppa)
+            doc.add_value(XapianValues.ARCHIVE_PPA, archive_ppa)
             # add archive origin data here so that its available even if
             # the PPA is not (yet) enabled
             doc.add_term("XOO"+"lp-ppa-%s" % archive_ppa.replace("/", "-"))
         # screenshot (for third party)
         if parser.has_option_desktop("X-AppInstall-Screenshot-Url"):
             url = parser.get_desktop("X-AppInstall-Screenshot-Url")
-            doc.add_value(XAPIAN_VALUE_SCREENSHOT_URL, url)
+            doc.add_value(XapianValues.SCREENSHOT_URL, url)
         # thumbnail (for third party)
         if parser.has_option_desktop("X-AppInstall-Thumbnail-Url"):
             url = parser.get_desktop("X-AppInstall-Thumbnail-Url")
-            doc.add_value(XAPIAN_VALUE_THUMBNAIL_URL, url)
+            doc.add_value(XapianValues.THUMBNAIL_URL, url)
+        # icon (for third party)
+        if parser.has_option_desktop("X-AppInstall-Icon-Url"):
+            url = parser.get_desktop("X-AppInstall-Icon-Url")
+            doc.add_value(XapianValues.ICON_URL, url)
+            if not parser.has_option_desktop("X-AppInstall-Icon"):
+                doc.add_value(XapianValues.ICON, os.path.basename(url))
         # price (pay stuff)
         if parser.has_option_desktop("X-AppInstall-Price"):
             price = parser.get_desktop("X-AppInstall-Price")
-            doc.add_value(XAPIAN_VALUE_PRICE, price)
+            doc.add_value(XapianValues.PRICE, price)
             # since this is a commercial app, indicate it in the component value
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_SECTION, "commercial")
+            doc.add_value(XapianValues.ARCHIVE_SECTION, "commercial")
         # icon
         if parser.has_option_desktop("Icon"):
             icon = parser.get_desktop("Icon")
-            doc.add_value(XAPIAN_VALUE_ICON, icon)
+            doc.add_value(XapianValues.ICON, icon)
         # write out categories
         for cat in parser.get_desktop_categories():
             doc.add_term("AC"+cat.lower())
+        categories_string = ";".join(parser.get_desktop_categories())
+        doc.add_value(XapianValues.CATEGORIES, categories_string)
         for mime in parser.get_desktop_mimetypes():
             doc.add_term("AM"+mime.lower())
         # get type (to distinguish between apps and packages
@@ -616,24 +723,18 @@ def index_app_info_from_parser(parser, db, cache):
         # check gettext domain
         if parser.has_option_desktop("X-Ubuntu-Gettext-Domain"):
             domain = parser.get_desktop("X-Ubuntu-Gettext-Domain")
-            doc.add_value(XAPIAN_VALUE_GETTEXT_DOMAIN, domain)
-        # architecture
-        if parser.has_option_desktop("X-AppInstall-Architectures"):
-            arches = parser.get_desktop("X-AppInstall-Architectures")
-            doc.add_value(XAPIAN_VALUE_ARCHIVE_ARCH, arches)
-            if apt_pkg.config.find("Apt::Architecture") not in arches:
-                return
+            doc.add_value(XapianValues.GETTEXT_DOMAIN, domain)
         # Description (software-center extension)
         if parser.has_option_desktop("X-AppInstall-Description"):
             descr = parser.get_desktop("X-AppInstall-Description")
-            doc.add_value(XAPIAN_VALUE_SC_DESCRIPTION, descr)
+            doc.add_value(XapianValues.SC_DESCRIPTION, descr)
         # popcon
         # FIXME: popularity not only based on popcon but also
         #        on archive section, third party app etc
         if parser.has_option_desktop("X-AppInstall-Popcon"):
             popcon = float(parser.get_desktop("X-AppInstall-Popcon"))
             # sort_by_value uses string compare, so we need to pad here
-            doc.add_value(XAPIAN_VALUE_POPCON, 
+            doc.add_value(XapianValues.POPCON, 
                           xapian.sortable_serialise(popcon))
             global popcon_max
             popcon_max = max(popcon_max, popcon)
@@ -643,14 +744,14 @@ def index_app_info_from_parser(parser, db, cache):
         # the summary of the package
         if parser.has_option_desktop("Comment"):
             s = parser.get_desktop("Comment")
-            doc.add_value(XAPIAN_VALUE_SUMMARY, s)
+            doc.add_value(XapianValues.SUMMARY, s)
         elif parser.has_option_desktop("GenericName"):
             s = parser.get_desktop("GenericName")
             if s != name:
-                doc.add_value(XAPIAN_VALUE_SUMMARY, s)
+                doc.add_value(XapianValues.SUMMARY, s)
         elif pkgname in cache and cache[pkgname].candidate:
             s = cache[pkgname].candidate.summary
-            doc.add_value(XAPIAN_VALUE_SUMMARY, s)
+            doc.add_value(XapianValues.SUMMARY, s)
 
         # add packagename as meta-data too
         term_generator.index_text_without_positions(pkgname, WEIGHT_APT_PKGNAME)
@@ -691,8 +792,10 @@ def index_app_info_from_parser(parser, db, cache):
         # now add it
         db.add_document(doc)
 
-def rebuild_database(pathname):
-    cache = apt.Cache(memonly=True)
+def rebuild_database(pathname, debian_sources=True, appstream_sources=False):
+    #cache = apt.Cache(memonly=True)
+    cache = get_pkg_info()
+    cache.open()
     old_path = pathname+"_old"
     rebuild_path = pathname+"_rb"
     
@@ -724,7 +827,12 @@ def rebuild_database(pathname):
             
     # write it
     db = xapian.WritableDatabase(rebuild_path, xapian.DB_CREATE_OR_OVERWRITE)
-    update(db, cache)
+
+    if debian_sources:
+        update(db, cache)
+    if appstream_sources:
+        update_from_appstream_xml(db, cache)
+
     # write the database version into the filep
     db.set_metadata("db-schema-version", DB_SCHEMA_VERSION)
     # update the mo file stamp for the langpack checks
